# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2018-2025, by Samuel Williams.

require_relative "rest/version"
require_relative "rest/representation"
require_relative "rest/wrapper"
