# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2025, by Samuel Williams.

require_relative "wrapper/form"
require_relative "wrapper/json"
require_relative "wrapper/url_encoded"
